<?php

/*  Copyright 2011 Simon Wheatley

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

require_once( 'class-Plugin.php' );

/**
 * This class controls the WP UI, etc.
 * 
 * @package SI Events
 * @author Simon Wheatley
 **/
class MBE2S extends MBE2S_Plugin {

	/**
	 * The version of this file, for the purposes of rewrite
	 * refreshes, JS/CSS cache bursting, etc.
	 *
	 * @var int
	 **/
	protected $version;

	// HOOKS
	// =====
	
	/**
	 * Initiate!
	 *
	 * @return void
	 * @author Simon Wheatley
	 **/
	public function __construct() {
		$this->setup( 'mbe2s' );
		if ( is_admin() ) {
			$this->add_action( 'admin_init' );
			$this->add_action( 'admin_menu' );
			$this->add_action( 'load-settings_page_mbe2s_options', 'load_settings' );
			$this->add_action( 'load-subscribers_page_mbe2s_message', 'load_send_message' );
			$this->add_action( 'load-toplevel_page_mbe2s_sms', 'load_subscribers' );
			$this->add_action( 'wp_ajax_mbe2s_subscribe', 'ajax_subscribe' );
			$this->add_action( 'wp_ajax_nopriv_mbe2s_subscribe', 'ajax_subscribe' );
		}
		$this->add_action( 'init' );
		$this->add_action( 'mbe2s_import_providers', 'import_providers' );
		$this->add_action( 'transition_post_status', null, null, 3 );
		$this->add_filter( 'upload_mimes' );
		$this->version = 1;
	}

	/**
	 * A Cron hook to import the providers.
	 *
	 * @return void
	 * @author Simon Wheatley
	 **/
	public function import_providers() {
		// This could return a WP_Error, but we're going to ignore
		// it if it does.
		Email2SMS_Providers::import();
	}

	/**
	 * Hooks the init action to:
	 * * Process unsubscribe requests
	 *
	 * @return void
	 * @author Simon Wheatley
	 **/
	public function init() {
		$this->maybe_process_unsubscribe();
	}

	/**
	 * Hooks the admin_init action to 
	 *
	 * @return void
	 * @author Simon Wheatley
	 **/
	public function admin_init() {
		$suffix = defined( 'SCRIPT_DEBUG' ) && SCRIPT_DEBUG ? '.dev' : '';
		wp_enqueue_script( 'mbe2s-admin', $this->url( "/js/admin{$suffix}.js" ), array( 'jquery' ), $this->version );
		$localized = array( 
			'menu_label' => __( 'SMS', 'mbe2s' ), 
		);
		wp_localize_script( 'mbe2s-admin', 'mbe2s', $localized );
		wp_enqueue_style( 'mbe2s-admin', $this->url( "/css/admin{$suffix}.css" ), array(), $this->version, 'all' );
		Email2SMS_Subscribers::maybe_upgrade();
		Email2SMS_Providers::maybe_upgrade();
	}
	
	/**
	 * Hooks the WP admin_menu function to add in our top
	 * level menu and various sub-pages.
	 *
	 * @return void
	 * @author Simon Wheatley
	 **/
	public function admin_menu() {
		// Create a top level SMS menu item, and some sub-items
		add_object_page( __( 'SMS Subscribers', 'mbe2s' ), __( 'Subscribers', 'mbe2s' ), 'edit_users', 'mbe2s_sms', array( $this, 'subscribers_admin_page' ) );
		add_submenu_page( 'mbe2s_sms', __( 'Send SMS Message', 'mbe2s' ), __( 'Send Message', 'mbe2s' ), 'publish_posts', 'mbe2s_message', array( $this, 'message_admin_page' ) );
		add_submenu_page( 'mbe2s_sms', __( 'SMS Providers', 'mbe2s' ), __( 'Providers', 'mbe2s' ), 'manage_options', 'mbe2s_providers', array( $this, 'providers_admin_page' ) );
		// Settings go into the settings section
		add_options_page( __( 'SMS Settings', 'mbe2s' ), __( 'SMS' ), 'manage_options', 'mbe2s_options', array( $this, 'options_admin_page' ) );
	}

	/**
	 * Hooks the load action for the SMS Options page.
	 *
	 * @return void
	 * @author Simon Wheatley
	 **/
	public function load_settings() {
		$to_do = (bool) @ $_POST[ '_mbe2s_nonce' ];
		if ( ! $to_do )
			return;
		$action = @ $_POST[ 'action' ];
		// We check the admin referer and nonce in the methods called below
		if ( 'reload-providers' == $action )
			$this->reload_providers();
		else if ( 'save-settings' == $action )
			$this->save_settings();
		else if ( 'delete-uninstall' == $action )
			$this->confirm_delete_uninstall();
		else if ( 'confirm-delete-uninstall' == $action )
			$this->delete_uninstall();
		wp_redirect( admin_url( '/options-general.php?page=mbe2s_options' ) );
		exit;
	}

	/**
	 * Hooks the load action for the Send SMS Message page.
	 *
	 * @return void
	 * @author Simon Wheatley
	 **/
	public function load_send_message() {
		$to_do = (bool) @ $_POST[ '_mbe2s_nonce' ];
		if ( ! $to_do )
			return;
		check_admin_referer( 'mbe2s-send-message', '_mbe2s_nonce' );
		$url = admin_url( '/admin.php?page=mbe2s_message' );
		$sms_message = trim( @ $_POST[ 'mbe2s_sms_message' ] );
		if ( ! $sms_message ) {
			$this->set_admin_error( __( 'Please enter a message to send.', 'mbe2s' ) );
			wp_redirect( $url );
			exit;
		}
		$message = $this->parse_message( $sms_message );
		$sms = new Transport_Email2SMS();
		$sms->set_message( $message );
		$sms->send_to_all();
		$this->set_admin_notice( __( 'Your SMS message has been sent.', 'mbe2s' ) );
		wp_redirect( $url );
		exit;
	}

	/**
	 * Hooks the load action for the SMS Subscribers page.
	 *
	 * @return void
	 * @author Simon Wheatley
	 **/
	public function load_subscribers() {
		$to_do = (bool) @ $_POST[ '_mbe2s_nonce' ];
		if ( ! $to_do )
			return;
		check_admin_referer( 'subscriber_action', '_mbe2s_nonce' );
		$action = @ $_POST[ 'action' ];
		if ( $action == 'delete' ) {
			if ( $subscriber_ids = (array) $_POST[ 'subscribers' ] ) {
				$result = Email2SMS_Subscribers::delete( $subscriber_ids );
				if ( is_wp_error( $result ) )
					$this->set_admin_error( $result->get_error_message() );
				else if ( ! $result )
					$this->set_admin_error( sprintf( __( 'Deleted %d subscribers.', 'mbe2s' ), $result ) );
				else
					$this->set_admin_notice( sprintf( __( 'Deleted %d subscribers.', 'mbe2s' ), $result ) );
			} else {
				$this->set_admin_error( __( 'Please select some subscribers to delete.', 'mbe2s' ) );
			}
		} else {
			$this->set_admin_notice( sprintf( __( 'Please select an action.', 'mbe2s' ), $result ) );
		}
		$current_url = ( is_ssl() ? 'https://' : 'http://' ) . $_SERVER['HTTP_HOST'] . $_SERVER['REQUEST_URI'];
		wp_redirect( $current_url );
		exit;
	}

	/**
	 * Hooks the transition_post_status action to send notifications.
	 *
	 * @param string $new_status The new status
	 * @param string $old_status The previous status 
	 * @param object $post A WP Post
	 * @return void
	 * @author Simon Wheatley
	 **/
	public function transition_post_status( $new_status, $old_status, $post ) {
		if ( $new_status == 'publish' && $old_status != 'publish' && $this->get_option( 'new_post', 1 ) ) 
			$subscription = 'publish_post';
		// We're only bothered about things we have a subscription for.
		if ( ! isset( $subscription ) ) 
			return;
		// All ready, send the notifications
		// Queue up a message via Cron
		$message = $this->parse_message( $this->new_post_message(), $post );
		$sms = new Transport_Email2SMS();
		$sms->set_message( $message );
		$sms->send_to_all();
	}

	/**
	 * Hooks the WP wp_ajax_nopriv_mbe2s_subscribe action for some AJAX action.
	 *
	 * @return void
	 **/
	public function ajax_subscribe() {
		$is_ajax = (bool) @ $_POST[ 'is-ajax' ];
		$errors = array();

		// $submission = @ $_POST[ 'mbe2s-subscribe' ];
		// if ( $this->id != $submission )
		// 	return false;

		// Right. Let's process the heck outta this thing.

		$number = @ $_POST[ 'number' ];
		if ( ! $number )
			$errors[ 'number' ] = __( 'Please enter a mobile phone number.', 'mbe2s' );

		$provider_id = @ $_POST[ 'provider' ];
		if ( ! $provider_id )
			$errors[ 'provider' ] = __( 'Please select a provider.', 'mbe2s' );
		elseif ( $provider_id == 'none' )
			$errors[ 'provider' ] = __( 'We are unable to send SMS messages to your mobile provider', 'mbe2s' );

		$result = Email2SMS_Subscribers::insert( $number, $provider_id );
		if ( is_wp_error( $result ) )
			$errors[ 'generic' ] = $result->get_error_message();

		if ( $errors ) {
			$msg = "<div class='mbe2s-errors'>";
			if ( count( $errors ) == 1 )
				$msg .= '<p>' . __( 'Sorry, we were not able to process your subscription request for the following reason:' ) . '</p><ul>';
			else
				$msg .= '<p>' . __( 'Sorry, we were not able to process your subscription request for the following reasons:' ) . '</p><ul>';
			foreach ( $errors as $e ) {
				$msg .= "<li>$e</li>";
			}
			$msg .= '</ul></div>';

			echo $msg;
			exit;
		}

		echo "<p class='mbe2s-success'>Your subscription has been stored.</p>";
		exit;
	}

	/**
	 * Hooks the WP upload_mimes filter to allow CSVs to be uploaded.
	 *
	 * @param array $mimes The currently allowed mimetypes as an array, e.g.  
	 * @return The allowed mimetypes
	 * @author Simon Wheatley
	 **/
	public function upload_mimes( $mimes ) {
		$mimes = array_merge( $mimes, array( 'csv' => 'text/csv' ) );
		return $mimes;
	}

	// CALLBACKS
	// =========

	/**
	 * Callback function to print the HTML for the main SMS 
	 * admin options page.
	 *
	 * @return void
	 * @author Simon Wheatley
	 **/
	public function options_admin_page() {
		global $wpdb;
		$vars = array();
		// This option controls whether a new post is sent a 
		// notification, defaults to true
		$vars[ 'new_post' ] = $this->get_option( 'new_post', 1 );
		// This option dictates the message sent
		$vars[ 'new_post_message' ] = $this->new_post_message();
		// Trying to limit this query so it doesn't scan the entire table, does this work?
		$vars[ 'length_post_title' ] = (int) ceil( $wpdb->get_var( " SELECT AVG( CHAR_LENGTH( post_title ) ) FROM $wpdb->posts WHERE post_type = 'post' AND post_status = 'publish' LIMIT 500 " ) );
		$vars[ 'length_post_author' ] = (int) ceil( $wpdb->get_var( " SELECT AVG( CHAR_LENGTH( display_name ) ) FROM $wpdb->users LIMIT 500 " ) );
		$vars[ 'length_post_excerpt' ] = (int) ceil( $wpdb->get_var( " SELECT AVG( CHAR_LENGTH( post_excerpt ) ) FROM $wpdb->posts WHERE post_type = 'post' AND post_status = 'publish' LIMIT 500 " ) );
		// N.B. The following are not multibyte safe, but mb string functions aren't always available
		$vars[ 'length_post_url' ] = strlen( wp_get_shortlink( $wpdb->get_var( " SELECT ID FROM $wpdb->posts WHERE post_type = 'post' AND post_status = 'publish' " ) ) );
		
		$vars[ 'length_blog_name' ] = strlen( get_bloginfo( 'name' ) );
		$vars[ 'length_blog_url' ] = strlen( home_url() );
		$this->render_admin( 'options.php', $vars );
	}

	/**
	 * Callback function to print the HTML for the SMS 
	 * providers page.
	 *
	 * @return void
	 * @author Simon Wheatley
	 **/
	public function message_admin_page() {
		$vars = array();
		$vars[ 'length_blog_name' ] = strlen( get_bloginfo( 'name' ) );
		$vars[ 'length_blog_url' ] = strlen( home_url() );
		$this->render_admin( 'message.php', $vars );
	}

	/**
	 * Callback function to print the HTML for the SMS 
	 * providers page.
	 *
	 * @return void
	 * @author Simon Wheatley
	 **/
	public function providers_admin_page() {
		$vars = array();
		$vars[ 'providers' ] = Email2SMS_Providers::get_all( false );
		$this->render_admin( 'providers.php', $vars );
	}

	/**
	 * Callback function to print the HTML for the SMS 
	 * subscribers page.
	 *
	 * @return void
	 * @author Simon Wheatley
	 **/
	public function subscribers_admin_page() {
		if ( ! $page = (int) @ $_GET[ 'paged' ] )
			$page = 1;
		$page_size = 20;
		$total_items = Email2SMS_Subscribers::count();
		$total_pages = ceil( $total_items / $page_size );
		$vars = array();
		$vars[ 'page' ] = $page;
		$vars[ 'total_items' ] = $total_items;
		$vars[ 'total_pages' ] = $total_pages;
		$vars[ 'page_size' ] = $page_size;
		$vars[ 'subscribers' ] = Email2SMS_Subscribers::get_paged( $page, $page_size, false );
		$this->render_admin( 'subscribers.php', $vars );
	}

	// METHODS
	// =======

	/**
	 * Parses the message, replacing the special codes as appropriate.
	 *
	 * @param string $message The message, with special codes
	 * @param object $post The WP Post object relating to the message
	 * @return string The message, all nice and parsed
	 * @author Simon Wheatley
	 **/
	protected function parse_message( $message, $post = null ) {
		$author = new WP_User( $post->post_author );
		$search = array( '{blog_name}', '{blog_url}' );
		// Some search values not necessary without a post
		if ( ! is_null( $post ) )
			$search = array_merge( $search, array( '{post_title}', '{post_author}', '{post_url}', '{post_excerpt}' ) );
		$replace = array( get_bloginfo( 'name' ), home_url() );
		// Some replace values not necessary without a post
		if ( ! is_null( $post ) )
			$replace = array_merge( $replace, array( get_the_title( $post->ID ), $author->display_name, wp_get_shortlink( $post->ID ), $post->post_excerpt ) );
		return str_replace( $search, $replace, $message );
	}

	/**
	 * Provides the default message, if the user hasn't specified one.
	 *
	 * @return string The default message
	 * @author Simon Wheatley
	 **/
	protected function new_post_message() {
		$default = sprintf( __( 'New %1$s post: %2$s', 'mbe2s' ), '{blog_name}', '{post_url}' );
		return stripslashes( $this->get_option( 'new_post_message', $default ) );
	}

	/**
	 * Reload the providers remotely from the Mediaburst CSV.
	 *
	 * @return void
	 * @author Simon Wheatley
	 **/
	protected function reload_providers() {
		check_admin_referer( 'mbe2s-reload-providers', '_mbe2s_nonce' );
		$result = Email2SMS_Providers::import();
		if ( is_wp_error( $result ) ) {
			$this->set_admin_error( $result->get_error_message() );
			error_log( "MBE2S ERROR: " . print_r( $result, true ) );
		} else {
			if ( Email2SMS_Providers::DATA_URL == apply_filters( 'mbe2s_providers_csv_url', Email2SMS_Providers::DATA_URL ) )
				$this->set_admin_notice( sprintf( __( 'The <a href="%s">SMS providers</a> have been reloaded from Mediaburst.', 'mbe2s' ), admin_url( 'admin.php?page=mbe2s_providers' ) ) );
			else
				$this->set_admin_notice( sprintf( __( 'The <a href="%1$s">SMS providers</a> have been reloaded from your custom location (<a href="%2$s">%2$s</a>).', 'mbe2s' ), admin_url( 'admin.php?page=mbe2s_providers' ), apply_filters( 'mbe2s_providers_csv_url', Email2SMS_Providers::DATA_URL ) ) );
		}
	}

	/**
	 * Save settings from the Options page.
	 *
	 * @return void
	 * @author Simon Wheatley
	 **/
	protected function save_settings() {
		check_admin_referer( 'mbe2s-save-settings', '_mbe2s_nonce' );
		$this->update_option( 'new_post', (bool) @ $_POST[ 'mbe2s_new_post' ] );
		$this->update_option( 'new_post_message', @ $_POST[ 'mbe2s_new_post_message' ] );
		$this->set_admin_notice( __( 'Settings saved.', 'mbe2s' ) );
	}

	/**
	 * Show a wp_die to confirm the user wants to delete all the plugin data.
	 *
	 * @return void
	 * @author Simon Wheatley
	 **/
	protected function confirm_delete_uninstall() {
		check_admin_referer( 'mbe2s-delete-uninstall', '_mbe2s_nonce' );
		if ( ! current_user_can( 'activate_plugins' ) )
			wp_die( __( 'Sorry, you are not allowed to deactivate plugins.', 'mbe2s' ) );
		$html = $this->capture_admin( 'confirm-delete-uninstall.php', array() );
		wp_die( $html, __( 'Confirm uninstall and delete', 'mbe2s' ), array( 'response' => 200 ) );
	}

	/**
	 * Show a wp_die to confirm the delete uninstall has happened.
	 *
	 * @return void
	 * @author Simon Wheatley
	 **/
	protected function delete_uninstall() {
		check_admin_referer( 'mbe2s-confirm-delete-uninstall', '_mbe2s_nonce' );
		if ( ! current_user_can( 'activate_plugins' ) )
			wp_die( __( 'Sorry, you are not allowed to deactivate plugins.', 'mbe2s' ) );
		Email2SMS_Providers::uninstall();
		Email2SMS_Subscribers::uninstall();
		delete_option( $this->name );
		deactivate_plugins( $this->folder . '/mediaburst-email-2-sms.php' );
		wp_die( sprintf( __( "The Mediaburst: Email to SMS plugin has been deactivated and all it's data deleted; return to the <a href='%s'>Dashboard</a>.", 'mbe2s' ), admin_url() ), __( 'Confirm uninstall and delete', 'mbe2s' ), array( 'response' => 200 ) );
	}


	/**
	 * Process any unsubscribe request.
	 *
	 * @return void
	 * @author Simon Wheatley
	 **/
	protected function maybe_process_unsubscribe() {
		$unsubscribe = @ $_REQUEST[ 'mbe2s-unsubscribe' ];
		if ( ! $unsubscribe )
			return;
		$vars = array();
		$removed = false;
		// Process any number sent
		if ( $vars[ 'number' ] = @ $_POST[ 'mbe2s-number' ] ) {
			$subscriber_id = Email2SMS_Subscribers::get_subscriber_by_number( $vars[ 'number' ] );
			if ( is_wp_error( $subscriber_id ) ) {
				$vars[ 'error' ] = $subscriber_id->get_error_message();
			} else {
				$removed = Email2SMS_Subscribers::delete( $subscriber_id );
				// Deal with any explicit error
				if ( is_wp_error( $removed ) )
					$vars[ 'error' ] = $removed->get_error_message();
				// It could be that the subscriber was never in our DB under that number
				if ( ! $removed )
					$vars[ 'error' ] = sprintf( __( 'We could not find a subscriber with the number you provided, which was: %s. You could add, or remove, any country code and try again. Sorry for any inconvenience.', 'mbe2s' ), $vars[ 'number' ] );
			}
		}
		if ( $removed ) {
			// If we've been POSTed a number, then show the confirmation view
			$html = $this->capture( 'unsubscribed.php', $vars );
		} else {
			// …otherwise show the unsubscribe form
			$html = $this->capture( 'unsubscribe.php', $vars );
		}
		wp_die( $html, __( 'Unsubscribe from SMS', 'mbe2s' ), array( 'response' => 200 ) );
	}

} // END MBE2S class 

$mbe2s = new MBE2S();

?>