<?php

/*  Copyright 2011 Simon Wheatley

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

/**
 * Send SMS messages using an Email to SMS gateway.
 * 
 * @package Mediaburst Email 2 SMS plugin
 * @author Simon Wheatley
 **/
class Email2SMS_Providers {

	/**
	 * The version of this class, for the purposes of rewrite
	 * refreshes, JS/CSS cache bursting, etc.
	 *
	 * @var int
	 **/
	const VERSION = 10;

	/**
	 * The URL for the CSV of providers information.
	 *
	 * @var string
	 **/
	const DATA_URL = 'http://www.mediaburst.co.uk/downloads/email-to-sms.csv';

	/**
	 * Get all the providers.
	 *
	 * @param boolean $active_only If true, return only active providers
	 * @return void
	 * @author Simon Wheatley
	 **/
	public static function get_all( $active_only = true ) {
		global $wpdb;
		$table = $wpdb->prefix . 'mb_providers';
		$where = '';
		if ( $active_only )
			$where = " AND provider_active = 1 ";
		$sql = " SELECT * FROM $table WHERE 1 = 1 $where ORDER BY provider_country, provider_name ASC ";
		return $wpdb->get_results( $sql );
	}

	/**
	 * Imports the providers into the DB.
	 *
	 * @return void
	 * @author Simon Wheatley
	 **/
	public static function import() {
		global $wpdb;

		// We are going to download the remote file to the local filesystem, 
		// then open and loop over it's contents.

		// Create an empty placeholder file in the uploads dir
		$upload = wp_upload_bits( 'mbe2s.csv', 0, '' );
		if ( $upload[ 'error' ] ) {
			return new WP_Error( 'mbe2s_dir_error', sprintf( __( 'Problem creating local file: %s' ), $upload[ 'error' ] ) );
		}

		// fetch the remote url and write it to the placeholder file
		$data_url = apply_filters( 'mbe2s_providers_csv_url', self::DATA_URL );
		$headers = wp_get_http( $data_url, $upload[ 'file' ] );

		//Request failed
		if ( ! $headers ) {
			@unlink( $upload[ 'file' ] );
			return new WP_Error( 'mbe2s_remote_error', __( 'Remote server did not respond', 'mbe2s' ) );
		}

		// make sure the fetch was successful
		if ( $headers['response'] != '200' ) {
			@unlink($upload['file']);
			return new WP_Error( 'mbe2s_remote_error', sprintf(__('Remote file returned error response %1$d %2$s', 'wordpress-importer'), $headers['response'], get_status_header_desc($headers['response']) ) );
		}
		elseif ( isset($headers['content-length']) && filesize($upload['file']) != $headers['content-length'] ) {
			@unlink($upload['file']);
			return new WP_Error( 'mbe2s_remote_error', __('Remote file is incorrect size', 'wordpress-importer') );
		}

		// Attempt to get around line ending issues, note that this might not work
		ini_set( 'auto_detect_line_endings', true );
		$file = fopen( $upload[ 'file' ], 'r' );
		$row = 0;
		$table = $wpdb->prefix . 'mb_providers';

		// We will store the inserts until we've validated the CSV, and only insert them
		// we're sure it's good to go.
		$inserts = array();
		$insert_sql = " INSERT INTO $table SET provider_id = %s, provider_active = %d, provider_name = %s, provider_country = %s, provider_domain = %s, provider_link = %s, provider_prefix = %s ";
		// Loop over the entire CSV
		$provider_ids = array();
		while( ( $data = fgetcsv( $file ) ) !== false ) {

			$row++;

			// First row is the headers, ignore it
			if ( $row == 1 )
				continue;

			list( $active, $id, $name, $country, $domain, $link, $prefix ) = $data;

			// Little bit of data cleansing and neatening
			$active = (int) $active;
			$id = trim( $id );
			$link = esc_url( $link );

			// Validate the row for required information (mainly for the benefit of Mediaburst post-upload check)
			if ( ! $id )
				return new WP_Error( 'mbe2s_provider_validation', sprintf( __( 'Column "ID" was blank in row %d', 'mbe2s' ), $row ) );
			if ( ! $name )
				return new WP_Error( 'mbe2s_provider_validation', sprintf( __( 'Column "name" was blank in row %d', 'mbe2s' ), $row ) );
			if ( ! $country )
				return new WP_Error( 'mbe2s_provider_validation', sprintf( __( 'Column "country" was blank in row %d', 'mbe2s' ), $row ) );
			 if ( ! $domain && $active )
				return new WP_Error( 'mbe2s_provider_validation', sprintf( __( 'Column "domain" was blank in row %d', 'mbe2s' ), $row ) );
				
			// Save the insert/update statement for later
			$inserts[] = $wpdb->prepare( $insert_sql, $id, $active, $name, $country, self::no_whitespace( $domain ), $link, self::no_whitespace( $prefix ) );
			
			if ( in_array( $id, $provider_ids ) )
				return new WP_Error( 'mbe2s_provider_validation', sprintf( __( 'Duplicate provider ID (%1$s) on row %2$d', 'mbe2s' ), $id, $row ) );
			$provider_ids[] = $id;
		}
		
		// If we never got past the first row, we could have a line endings encoding issue
		// so don't proceed with this data and have a whinge about it
		if ( $row == 0 )
			return new WP_Error( 'mbe2s_no_providers', __( "The providers CSV had no entries (possible line endings issue).", 'mbe2s' ) );

		// Clean the providers from the table
		if ( false === $wpdb->query( "TRUNCATE $table"  ) ) {
				// Attempt to capture any WPDB errors, this will only capture
				// on single WP installs, which echo the error, network installs 
				// will send the error to the PHP error log.
				ob_start();
				$wpdb->print_error();
				$error = strip_tags( ob_get_contents() );
				ob_end_clean();
				error_log( "MBE2S WPDB error: $error" );
				return new WP_Error( 'mbe2s_insert_providers', sprintf( __( 'Was not able to clean the providers table (DB error, if available: %s)', 'mbe2s' ), $error ) );
		}

		// Now we're happy, run the insert/update statements
		foreach ( $inserts as $sql ) {
			if ( false === $wpdb->query( $sql  ) ) {
				// Attempt to capture any WPDB errors, this will only capture
				// on single WP installs, which echo the error, network installs 
				// will send the error to the PHP error log.
				ob_start();
				$wpdb->print_error();
				$error = strip_tags( ob_get_contents() );
				ob_end_clean();
				error_log( "MBE2S WPDB error: $error" );
				return new WP_Error( 'mbe2s_insert_providers', sprintf( __( 'Was not able to insert provider from row %1$d (DB error, if available: %2$s)', 'mbe2s' ), $i+1, $error ) );
			}
		}

		// Clean up after ourselves
		fclose( $file );
		unlink( $upload[ 'file' ] );
	}

	/**
	 * Removes whitespace from a string
	 *
	 * @param string $str A string to remove whitespace from 
	 * @return string A string with no whitespace
	 **/
	public static function no_whitespace( $str ) {
		return preg_replace( '/\s/', '', $str );
	}

	/**
	 * Checks the DB structure is up to date.
	 *
	 * @return void
	 * @author Simon Wheatley
	 **/
	public static function maybe_upgrade() {
		global $wpdb;
		$version = get_option( 'mbe2s-providers-version', 0 );
		
		$done_upgrade = false;

		// N.B. Remember to increment self::VERSION when you add a new IF
		if ( self::VERSION == $version )
			return;

		if ( $version < 2 ) {
			error_log( "MBE2S: Setting cron schedule" );
			wp_clear_scheduled_hook( 'mbe2s_import_providers' );
			wp_schedule_event( time(), 'daily', 'mbe2s_import_providers' );
		}

		if ( $version < 9 ) {
			error_log( "MBE2S: Upgrading providers DB table" );
			$charset_collate = mbe2s_db_charset_collate();
			$table = $wpdb->prefix . 'mb_providers';
			$wpdb->query( "DROP TABLE $table" );
			$sql  = " CREATE TABLE $table ( ";
			$sql .= "    provider_id VARCHAR(20) NOT NULL, ";
			$sql .= "    provider_active TINYINT NOT NULL DEFAULT 0, ";
			$sql .= "    provider_name VARCHAR(200) NOT NULL, ";
			$sql .= "    provider_country VARCHAR(100) NOT NULL, ";
			$sql .= "    provider_domain VARCHAR(100) NOT NULL, ";
			$sql .= "    provider_link VARCHAR(100) NOT NULL, ";
			$sql .= "PRIMARY KEY ( provider_id ) ) $charset_collate ";
			$wpdb->query( $sql );
			// We dropped the table, so (attempt to) now repopulate it
			self::import();
			$done_upgrade = true;
		}

		if ( $version < 10 ) {
			error_log( "MBE2S: Upgrading providers DB table to add prefix column" );
			$charset_collate = mbe2s_db_charset_collate();
			$table = $wpdb->prefix . 'mb_providers';
			$sql = " ALTER TABLE $table ADD provider_prefix VARCHAR(100) ";
			$wpdb->query( $sql );
			// We dropped the table, so (attempt to) now repopulate it
			self::import();
		}

		error_log( "MBE2S: Done upgrade" );
		update_option( 'mbe2s-providers-version', self::VERSION );
	}
	
	/**
	 * Delete the data.
	 *
	 * @return void
	 * @author Simon Wheatley
	 **/
	public static function uninstall() {
		global $wpdb;
		$table = $wpdb->prefix . 'mb_providers';
		$wpdb->query( "DROP TABLE $table" );
		delete_option( 'mbe2s-providers-version' );
	}

} // END Email2SMS_Providers class 

?>
