<?php

/*  Copyright 2011 Simon Wheatley

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

// require_once( 'class-SMS.php' );

/**
 * Send SMS messages using an Email to SMS gateway.
 * 
 * @package Mediaburst Email 2 SMS plugin
 * @author Simon Wheatley
 **/
class Email2SMS_Subscribers {

	/**
	 * The version of this class, for the purposes of rewrite
	 * refreshes, JS/CSS cache bursting, etc.
	 *
	 * @var int
	 **/
	const VERSION = 7;

	/**
	 * Insert a subscriber, storing their mobile number against a provider ID.
	 *
	 * @param string $number The mobile phone number of the subscriber
	 * @param int $provider_id The ID of the provider to assign to this number
	 * @return void
	 * @author Simon Wheatley
	 **/
	public static function insert( $number, $provider_id ) {
		global $wpdb;
		$table = $wpdb->prefix . 'mb_subscribers';
		$number = self::clean_number( $number );
		// Check we don't already have a subscriber
		if ( self::get_subscriber_by_number( $number ) )
			return new WP_Error( 'mbe2s_duplicate', sprintf( __( 'There is already a subscriber with that mobile number, if you are changing providers, please <a href="%s">unsubscribe</a> first.' ), add_query_arg( array( 'mbe2s-unsubscribe' => 1 ), home_url() ) ) );
		$check_sql = " SELECT subscriber_id FROM $table WHERE provider_id = %s AND subscriber_number = %s ";
		if ( $wpdb->get_var( $wpdb->prepare( $check_sql, $provider_id, $number ) ) )
			return;
		$data = array(
			'provider_id' => $provider_id,
			'subscriber_number' => $number,
		);
		return $wpdb->insert( $table, $data );
	}

	/**
	 * Get the subscribers in batches.
	 *
	 * @param int $page The current page
	 * @param int $page_size The number of items on a page
	 * @param string $has_provider Whether a subscriber has a current provider or not. If null, all subscribers are returned [@TODO]
	 * @return array An array of Subscriber objects
	 * @author Simon Wheatley
	 **/
	public static function get_paged( $page, $page_size, $has_provider = true ) {
		global $wpdb;
		$start = ( $page - 1 ) * $page_size;
		$subscribers = $wpdb->prefix . 'mb_subscribers';
		$providers = $wpdb->prefix . 'mb_providers';
		$where = '';
		if ( $has_provider )
			$where = ' AND provs.provider_active = 1 ';
		$prepared_sql = $wpdb->prepare( " SELECT * FROM $subscribers AS subs LEFT OUTER JOIN $providers AS provs ON subs.provider_id = provs.provider_id WHERE 1 = 1 $where ORDER BY subs.provider_id, subs.subscriber_number ASC LIMIT %d, %d ", $start, $page_size );
		return $wpdb->get_results( $prepared_sql );
	}
	
	/**
	 * Delete subscribers as per the IDs.
	 *
	 * @param int|array $subscriber_ids The ID(s) of the subscribers to delete 
	 * @return int|object Either the number of subscribers deleted, or a WP_Error
	 * @author Simon Wheatley
	 **/
	public static function delete( $subscriber_ids ) {
		global $wpdb;
		if ( ! is_array( $subscriber_ids ) )
			$subscriber_ids = (array) $subscriber_ids;
		$table = $wpdb->prefix . 'mb_subscribers';
		// Sanitise the inputs, as we can't use prepare in this instance
		$subscriber_ids = array_map( 'absint', $subscriber_ids );
		$subscribers = join( ', ', $subscriber_ids );
		$sql = " DELETE FROM $table WHERE subscriber_id IN ( $subscribers ) ";
		if ( $deleted = $wpdb->query( $sql ) ) {
			if ( false === $deleted ) {
				ob_start();
				$wpdb->print_error();
				$error = ob_get_contents();
				ob_end_clean();
				return new WP_Error( 'mbe2s-delete-subscriber', $error );
			}
		}
		return (int) $deleted;
	}

	/**
	 * Returns the ID associated with the number given.
	 *
	 * @param string $number The mobile number to search on 
	 * @return int The ID of the subscriber, or null on failure
	 * @author Simon Wheatley
	 **/
	public static function get_subscriber_by_number( $number ) {
		global $wpdb;
		$number = self::clean_number( $number );
		$table = $wpdb->prefix . 'mb_subscribers';
		return $wpdb->get_var( $wpdb->prepare( " SELECT subscriber_id FROM $table WHERE subscriber_number = %s ", $number ) );
	}

	/**
	 * Count the subscribers.
	 *
	 * @return int The number of subscribers
	 * @author Simon Wheatley
	 **/
	public static function count() {
		global $wpdb;
		$subscribers = $wpdb->prefix . 'mb_subscribers';
		$sql = " SELECT COUNT( * ) FROM $subscribers ";
		return (int) $wpdb->get_var( $sql );
	}

	/**
	 * Cleans a mobile number for insertion into the DB, removes
	 * whitespace and unneeded characters.
	 *
	 * @param string $number The number to clean 
	 * @return string The clean number
	 * @author Simon Wheatley
	 **/
	protected static function clean_number( $number ) {
		// Remove whitespace from $number to canonicalise it
		$number = preg_replace( '/[\s]/i', '', $number );
		// Remove non-numerical and "+" characters from $number
		$number = preg_replace( '/[^0-9|^\+]+/i', '', $number );
		// Remove the crazy hat character
		$number = str_replace( '^', '', $number );
		return $number;
	}

	/**
	 * Checks the DB structure is up to date.
	 *
	 * @return void
	 * @author Simon Wheatley
	 **/
	public static function maybe_upgrade() {
		global $wpdb;
		$version = get_option( 'mbe2s-subscribers-version', 0 );
		
		$done_upgrade = false;

		// N.B. Remember to increment self::VERSION when you add a new IF, as otherwise
		// that upgrade will run on all admin loads!

		if ( $version < 7 ) {
			error_log( "MBE2S: Upgrading subscribers DB table" );
			$charset_collate = mbe2s_db_charset_collate();
			$table = $wpdb->prefix . 'mb_subscribers';
			$wpdb->query( "DROP TABLE $table" );
			$sql  = " CREATE TABLE $table ( ";
			$sql .= "    subscriber_id BIGINT(20) UNSIGNED NOT NULL AUTO_INCREMENT, ";
			$sql .= "    provider_id VARCHAR(20) NULL, ";
			$sql .= "    subscriber_number VARCHAR(200) NOT NULL, ";
			$sql .= " PRIMARY KEY ( subscriber_id ), ";
			$sql .= " UNIQUE KEY subscriber_number ( subscriber_number ) ) $charset_collate ";
			$wpdb->query( $sql );
			$done_upgrade = true;
		}

		if ( $done_upgrade ) {
			error_log( "MBE2S: Done upgrade" );
			update_option( 'mbe2s-subscribers-version', self::VERSION );
		}
	}
	
	/**
	 * Delete the data.
	 *
	 * @return void
	 * @author Simon Wheatley
	 **/
	public static function uninstall() {
		global $wpdb;
		$table = $wpdb->prefix . 'mb_subscribers';
		$wpdb->query( "DROP TABLE $table" );
		delete_option( 'mbe2s-subscribers-version' );
	}

} // END Email2SMS_Subscribers class 

?>